<?php
class pluginRedirectPages extends Plugin {
    public function init() {
        $this->dbFields = [
            "force-https" => false,
            "https-status" => 301,
            "allow-external" => false,
            "redirects" => []
        ];
        return true;
    }
    public function beforeAll() {
        global $url;
        $redirects = $this->getValue("redirects");
        $currentSlug = rtrim($url->slug(), '/');
        $allowExternal = $this->getValue("allow-external");
        if (isset($redirects[$currentSlug])) {
            $to = $this->formRedirectUrl($redirects[$currentSlug]['to'], $allowExternal);
            $status = $redirects[$currentSlug]['status'];
            if (!empty($_GET)) {
                $to .= "?" . http_build_query($_GET);
            }
            header("HTTP/1.1 " . ($status === 301 ? "301 Moved Permanently" : "302 Found"));
            header("Location: $to");
            exit;
        }
    }
    private function formRedirectUrl($to, $allowExternal) {
        if (strpos($to, "http://") === 0 || strpos($to, "https://") === 0) {
            if ($allowExternal) {
                return $to;
            }
        }
        return rtrim(DOMAIN_BASE, '/') . '/' . ltrim($to, '/');
    }
    public function form() {
        global $site;
        $redirects = $this->getValue("redirects");
        ?>
        <div class="form-group row">
            <div class="col-3 pt-3">
                <label for="force-https" class="text-right pt-2 mr-3" style="margin-top:0!important;">Force HTTPS</label>
            </div>
            <div class="col-9 pt-2 pl-4">
                <div class="custom-control custom-checkbox">
                    <input type="hidden" name="force-https" value="false" />
                    <input id="force-https" type="checkbox" name="force-https" value="true" class="custom-control-input" <?php if($this->getValue("force-https")) echo 'checked'; ?> />
                    <label class="custom-control-label" for="force-https">Force HTTPS (SSL) Everywhere</label>
                </div>
            </div>
        </div>
        <div class="form-group row">
            <div class="col-3 pt-3">
                <label for="https-status" class="text-right pt-2 mr-3" style="margin-top:0!important;">Redirection Type</label>
            </div>
            <div class="col-9 pt-3">
                <select id="https-status" name="https-status" class="custom-select">
                    <option value="301" <?php if($this->getValue("https-status") == 301) echo 'selected'; ?>>301 - Moved Permanently</option>
                    <option value="302" <?php if($this->getValue("https-status") == 302) echo 'selected'; ?>>302 - Moved Temporarily</option>
                </select>
            </div>
        </div>
        <div class="form-group row">
            <div class="col-3 pt-3">
                <label for="allow-external" class="text-right pt-2 mr-3" style="margin-top:0!important;">External Redirects</label>
            </div>
            <div class="col-9 pt-2 pl-4">
                <div class="custom-control custom-checkbox">
                    <input type="hidden" name="allow-external" value="false" />
                    <input id="allow-external" type="checkbox" name="allow-external" value="true" class="custom-control-input" <?php if($this->getValue("allow-external")) echo 'checked'; ?> />
                    <label class="custom-control-label" for="allow-external">Allow External Redirects</label>
                </div>
            </div>
        </div>
        <div class="col-12 mt-4 mb-3"><hr /></div>
        <h3>Redirect Rules</h3>
        <div id="redirects-list">
            <?php foreach ($redirects as $from => $data): ?>
                <div class="redirect-rule">
                    Page Slug (Friendly URL): <input type="text" name="redirects[<?php echo htmlspecialchars($from); ?>][old]" value="<?php echo htmlspecialchars($from); ?>" />
                    To URL / Slug: <input type="text" name="redirects[<?php echo htmlspecialchars($from); ?>][new]" value="<?php echo htmlspecialchars($data['to']); ?>" />
                    Redirection Type: <select name="redirects[<?php echo htmlspecialchars($from); ?>][status]">
                        <option value="301" <?php if ($data['status'] == 301) echo 'selected'; ?>>301 - Moved Permanently</option>
                        <option value="302" <?php if ($data['status'] == 302) echo 'selected'; ?>>302 - Moved Temporarily</option>
                    </select>
                    <button type="button" onclick="removeRedirect(this);" class="btn btn-danger btn-sm mt-2">Remove Redirect</button>
                </div>
            <?php endforeach; ?>
        </div>
        <div class="col-12 mt-4 mb-3"><hr /></div>
        <button type="button" onclick="addRedirect();" class="btn btn-primary btn-sm mt-2 float-right">Add New Redirect</button>
        <script>
        function addRedirect() {
            var container = document.getElementById('redirects-list');
            var count = container.children.length;
            var div = document.createElement('div');
            div.className = 'redirect-rule';
            div.innerHTML = `Page Slug (Friendly URL): <input type="text" name="redirects[new${count}][old]" />
                            To URL / Slug: <input type="text" name="redirects[new${count}][new]" />
                            Redirection Type: <select name="redirects[new${count}][status]">
                                        <option value="301">301 - Moved Permanently</option>
                                        <option value="302">302 - Moved Temporarily</option>
                                    </select>
                            <button type="button" onclick="removeRedirect(this);" class="btn btn-danger btn-sm mt-2">Remove Redirect</button>`;
            container.appendChild(div);
        }

        function removeRedirect(button) {
            button.parentNode.remove();
        }
        </script>
        <?php
    }
    public function post() {
        $this->db["force-https"] = $_POST["force-https"] === "true";
        $this->db["https-status"] = (int)$_POST["https-status"];
        $this->db["allow-external"] = $_POST["allow-external"] === "true";

        $redirects = [];
        foreach ($_POST["redirects"] as $from => $data) {
            if (trim($data['old']) !== '' && trim($data['new']) !== '') {
                $redirects[trim($data['old'])] = ['to' => trim($data['new']), 'status' => (int)$data['status']];
            }
        }
        $this->db["redirects"] = $redirects;
        return $this->save();
    }
    public function adminSidebar() {
        return '<a href="' . HTML_PATH_ADMIN_ROOT . 'configure-plugin/pluginRedirectPages" class="nav-link">Redirects</a>';
    }
}